@@ -9,7 +9,10 @@ NULL
 #' the expression is described in \code{\link{syntax}}. 
 #' 
 #' @param ... A comma-separated list of validating expressions
-#' @param .file A character vector of file locations (see also the section on file parsing in the 
+#' @param .file (optional) A character vector of file locations (see also the
+#'   section on file parsing in the
+#' @param .data (optional) A \code{data.frame} with columns \code{"rule"},
+#'   \code{"name"}, and \code{"description"}
 #' \code{\link{syntax}} help file).
 #'
 #' @section Validating expressions:
@@ -30,7 +33,8 @@ NULL
 #'
 #' @example ../examples/validator.R
 #' @export
-validator <- function(..., .file) new('validator',..., .file = .file)
+validator <- function(..., .file, .data) new('validator',...
+            , .file = .file, .data=.data)
 
 #### VALIDATOR CLASS ----------------------------------------------------------
 
@@ -58,16 +62,16 @@ validator <- function(..., .file) new('validator',..., .file = .file)
 setRefClass("validator"
   , contains = 'expressionset'
   , methods = list(
-    initialize = function(..., .file)  ini_validator(.self,...,.file=.file)
+    initialize = function(..., .file, .data)  ini_validator(.self,...,.file=.file, .data=.data)
     , is_linear = function() linear(.self)
       # extra argument: normalize=TRUE
     , linear_coefficients = function(...) get_linear_coefficients(.self, ...) 
   )
 )
 
-ini_validator <- function(obj, ..., .file){
+ini_validator <- function(obj, ..., .file, .data){
   check_primitives()
-  if (missing(.file)){
+  if (missing(.file) && missing(.data) ){
     .ini_expressionset_cli(obj, ..., .prefix="V")
     obj$._options <- .PKGOPT
     i <- validating(obj) | is_tran_assign(obj)
@@ -80,8 +84,17 @@ ini_validator <- function(obj, ..., .file){
         ))
       obj$rules <- obj$rules[i]
     } 
-  } else {
+  } else if (!missing(.file)) {
     .ini_expressionset_yml(obj, file=.file, .prefix="V")
+  } else if (!missing(.data)){
+    .ini_expressionset_df(obj, dat=.data, .prefix="V")
+    i <- validating(obj) | is_tran_assign(obj)
+    if (!all(i)){
+      r <- paste(which(!i),collapse=", ")
+      warning("Invalid syntax detected, ignoring rows ",r)
+      obj$rules <- obj$rules[i]
+    }
+    obj$._options <- .PKGOPT
   }
   # do options check.
 }
@@ -143,7 +156,29 @@ get_linear_coefficients <- function(x, normalize=TRUE,...){
 }
 
 
-
+#' Combine two validator objects
+#'
+#' Combine two \code{\link{validator}} objects by addition. A new \code{validator} 
+#' object is created with default (global) option values. Previously set options
+#' are ignored. 
+#'
+#' @param e1 a \code{\link{validator}}
+#' @param e2 a \code{\link{validator}}
+#'
+#' @section Note:
+#' The \code{names} of the resulting object are made unique using \code{\link[base]{make.names}}.
+#'
+#'
+#' @examples
+#' validator(x>0) + validator(x<=1)
+#'
+#' @export
+setMethod("+", c("validator","validator"), function(e1, e2){
+  v <- validator()
+  v$rules <- c(e1$rules, e2$rules)
+  names(v) <- make.names(names(v),unique=TRUE)
+  v
+})
 
 
 
